package org.jeecg.modules.gis.lochistory.controller;

import java.util.Arrays;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.modules.gis.lochistory.entity.LocHistory;
import org.jeecg.modules.gis.lochistory.service.ILocHistoryService;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.extern.slf4j.Slf4j;

import org.jeecg.common.system.base.controller.JeecgController;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.ModelAndView;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.jeecg.common.aspect.annotation.AutoLog;

 /**
 * @Description: 定位信息表
 * @Author: jeecg-boot
 * @Date:   2022-07-01
 * @Version: V1.0
 */
@Api(tags="定位信息表")
@RestController
@RequestMapping("/gis.lochistory/locHistory")
@Slf4j
public class LocHistoryController extends JeecgController<LocHistory, ILocHistoryService> {
	@Autowired
	private ILocHistoryService locHistoryService;
	
	/**
	 * 分页列表查询
	 *
	 * @param locHistory
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	//@AutoLog(value = "定位信息表-分页列表查询")
	@ApiOperation(value="定位信息表-分页列表查询", notes="定位信息表-分页列表查询")
	@GetMapping(value = "/list")
	public Result<IPage<LocHistory>> queryPageList(LocHistory locHistory,
								   @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
								   @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
								   HttpServletRequest req) {
		QueryWrapper<LocHistory> queryWrapper = QueryGenerator.initQueryWrapper(locHistory, req.getParameterMap());
		Page<LocHistory> page = new Page<LocHistory>(pageNo, pageSize);
		IPage<LocHistory> pageList = locHistoryService.page(page, queryWrapper);
		return Result.OK(pageList);
	}
	
	/**
	 *   添加
	 *
	 * @param locHistory
	 * @return
	 */
	@AutoLog(value = "定位信息表-添加")
	@ApiOperation(value="定位信息表-添加", notes="定位信息表-添加")
	@PostMapping(value = "/add")
	public Result<String> add(@RequestBody LocHistory locHistory) {
		locHistoryService.save(locHistory);
		return Result.OK("添加成功！");
	}
	
	/**
	 *  编辑
	 *
	 * @param locHistory
	 * @return
	 */
	@AutoLog(value = "定位信息表-编辑")
	@ApiOperation(value="定位信息表-编辑", notes="定位信息表-编辑")
	@RequestMapping(value = "/edit", method = {RequestMethod.PUT,RequestMethod.POST})
	public Result<String> edit(@RequestBody LocHistory locHistory) {
		locHistoryService.updateById(locHistory);
		return Result.OK("编辑成功!");
	}
	
	/**
	 *   通过id删除
	 *
	 * @param id
	 * @return
	 */
	@AutoLog(value = "定位信息表-通过id删除")
	@ApiOperation(value="定位信息表-通过id删除", notes="定位信息表-通过id删除")
	@DeleteMapping(value = "/delete")
	public Result<String> delete(@RequestParam(name="id",required=true) String id) {
		locHistoryService.removeById(id);
		return Result.OK("删除成功!");
	}
	
	/**
	 *  批量删除
	 *
	 * @param ids
	 * @return
	 */
	@AutoLog(value = "定位信息表-批量删除")
	@ApiOperation(value="定位信息表-批量删除", notes="定位信息表-批量删除")
	@DeleteMapping(value = "/deleteBatch")
	public Result<String> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		this.locHistoryService.removeByIds(Arrays.asList(ids.split(",")));
		return Result.OK("批量删除成功!");
	}
	
	/**
	 * 通过id查询
	 *
	 * @param id
	 * @return
	 */
	//@AutoLog(value = "定位信息表-通过id查询")
	@ApiOperation(value="定位信息表-通过id查询", notes="定位信息表-通过id查询")
	@GetMapping(value = "/queryById")
	public Result<LocHistory> queryById(@RequestParam(name="id",required=true) String id) {
		LocHistory locHistory = locHistoryService.getById(id);
		if(locHistory==null) {
			return Result.error("未找到对应数据");
		}
		return Result.OK(locHistory);
	}

    /**
    * 导出excel
    *
    * @param request
    * @param locHistory
    */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, LocHistory locHistory) {
        return super.exportXls(request, locHistory, LocHistory.class, "定位信息表");
    }

    /**
      * 通过excel导入数据
    *
    * @param request
    * @param response
    * @return
    */
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        return super.importExcel(request, response, LocHistory.class);
    }

}
