package org.jeecg.modules.resManager.catalog.controller;

import java.util.*;
import java.util.stream.Collectors;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.constant.CommonConstant;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecg.common.system.vo.SelectTreeModel;

import org.jeecg.modules.resManager.catalog.entity.Rescatalog;
import org.jeecg.modules.resManager.catalog.service.IRescatalogService;
import org.jeecg.modules.base.model.TreeModel;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.extern.slf4j.Slf4j;

import org.jeecg.modules.resManager.categories.entity.Layercategories;
import org.jeecg.modules.resManager.categories.service.ILayercategoriesService;
import org.jeecg.modules.resManager.fields.entity.Layerfields;
import org.jeecg.modules.resManager.fields.service.ILayerfieldsService;
import org.jeecg.modules.topicManage.entity.TopicsConfig;
import org.jeecg.modules.topicManage.service.ITopicsConfigService;
import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.jeecg.common.system.base.controller.JeecgController;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;
import com.alibaba.fastjson.JSON;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.jeecg.common.aspect.annotation.AutoLog;

/**
 * @Description: rescatalog
 * @Author: jeecg-boot
 * @Date: 2022-07-06
 * @Version: V1.0
 */
@Api(tags = "rescatalog")
@RestController
@RequestMapping("/resManager.catalog/rescatalog")
@Slf4j
public class RescatalogController extends JeecgController<Rescatalog, IRescatalogService> {
    @Autowired
    private IRescatalogService rescatalogService;
    @Autowired
    private ITopicsConfigService topicsConfigService;

    //	 @Autowired
    //	 private ILayercategoriesService layercategoriesService;
    //	 @Autowired
    //	 private ILayerfieldsService layerfieldsService;

    @AutoLog(value = "资源目录-用户资源目录查询")
    @ApiOperation(value = "资源目录-用户资源目录查询", notes = "资源目录-用户资源目录查询")
    @GetMapping(value = "/resListByUser")
    public Result<?> getResListByUserName(@RequestParam("username") String username, @RequestParam("topicId") String topicId) {
        long start = System.currentTimeMillis();
        List<JSONObject> resList = new ArrayList<>();
        //List<Rescatalog> rescatalogList = rescatalogService.queryByUser(username);
        resList = getResListByUser("0", username, topicId);
        log.info("======用户资源目录查询成功=====耗时:" + (System.currentTimeMillis() - start) + "毫秒");
        return Result.OK(resList);
    }

    private List<JSONObject> getResListByUser(String pid, String username, String topicId) {
        List<JSONObject> resList = new ArrayList<>();
        LambdaQueryWrapper<Rescatalog> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(Rescatalog::getPid, pid).orderByAsc(Rescatalog::getDisplayorder);
        //查询pid=0的一级内容
        List<Rescatalog> resCatalogs = rescatalogService.list(queryWrapper);
        System.out.println("##################################################################################################");
        System.out.println(resCatalogs);
        if (resCatalogs.size() > 0) {
            String displayers = "";
            String analysistypes = "";
            //topicId is null
            if (!"null".equals(topicId) && !topicId.isEmpty()) {
                LambdaQueryWrapper<TopicsConfig> topicsConfigLambdaQueryWrapper = new LambdaQueryWrapper<>();
                topicsConfigLambdaQueryWrapper.eq(TopicsConfig::getTopicid, topicId);
                TopicsConfig topicsConfig = topicsConfigService.getOne(topicsConfigLambdaQueryWrapper);
                displayers = topicsConfig.getDisplayers() == null ? "" : topicsConfig.getDisplayers();
                analysistypes = topicsConfig.getAnalysistypes() == null ? "" : topicsConfig.getAnalysistypes();
            }
            for (int i = 0; i < resCatalogs.size(); i++) {
                JSONObject jsonObject = new JSONObject();
                Rescatalog rescatalog = resCatalogs.get(i);
                String res_id = rescatalog.getId();
                int resCount = rescatalogService.queryCountByUsername(username, res_id);
                if (resCount == 0) {
                    continue;
                }
                String has_child = rescatalog.getHasChild();
                //一级
                if (("1").equals(has_child)) {
                    //一级路径
                    jsonObject.put("id", rescatalog.getId());
                    jsonObject.put("name", rescatalog.getAlias());
                    jsonObject.put("displayorder", rescatalog.getDisplayorder());
                    jsonObject.put("layers", getResListByUser(rescatalog.getId(), username, topicId));
                } else {
                    //二级路径
                    jsonObject.put("id", rescatalog.getId());
                    jsonObject.put("name", rescatalog.getName());
                    jsonObject.put("alias", rescatalog.getAlias());
                    jsonObject.put("url", rescatalog.getUrl());
                    jsonObject.put("sublayerid", rescatalog.getSublayerid());
                    jsonObject.put("dataport", rescatalog.getDataport());
                    jsonObject.put("relfield", rescatalog.getRelfield());
                    jsonObject.put("legend", rescatalog.getLegend());
                    //样式名称为图层名称去掉工作区
                    jsonObject.put("style", rescatalog.getStyle());
                    if (!displayers.isEmpty() && displayers.indexOf(rescatalog.getId()) != -1) {
                        jsonObject.put("visible", true);
                    } else {
                        jsonObject.put("visible", Boolean.parseBoolean(rescatalog.getVisible()));
                    }
                    jsonObject.put("labelsvisible", Boolean.parseBoolean(rescatalog.getLabelsvisible()));
                    if (analysistypes.isEmpty()) {
                        jsonObject.put("analyzable", Boolean.parseBoolean(rescatalog.getAnalyzable()));
                    } else if (analysistypes.indexOf(rescatalog.getId()) != -1) {
                        jsonObject.put("analyzable", true);
                    } else {
                        jsonObject.put("analyzable", false);
                    }
                    jsonObject.put("sourceName", rescatalog.getSourcename());
                    jsonObject.put("queryable", Boolean.parseBoolean(rescatalog.getQueryable()));
                    jsonObject.put("tablename", rescatalog.getTablename());
                    jsonObject.put("servicetype", rescatalog.getServicetype());
                    jsonObject.put("zIndex", rescatalog.getZindex());
                    jsonObject.put("displayorder", rescatalog.getDisplayorder());
                }
                resList.add(jsonObject);
            }
        }
        return resList;
    }


    @AutoLog(value = "资源目录-资源目录查询")
    @ApiOperation(value = "资源目录-资源目录查询", notes = "资源目录-资源目录查询")
    @GetMapping(value = "/resList")
    public Result<?> getResList() {
        List<JSONObject> resList = new ArrayList<>();
        resList = getResListByPid("0");
        return Result.OK(resList);
    }

    private List<JSONObject> getResListByPid(String pid) {
        List<JSONObject> resList = new ArrayList<>();
        LambdaQueryWrapper<Rescatalog> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(Rescatalog::getPid, pid).orderByAsc(Rescatalog::getDisplayorder);
        List<Rescatalog> resCatalogs = rescatalogService.list(queryWrapper);
        if (resCatalogs.size() > 0) {
            for (int i = 0; i < resCatalogs.size(); i++) {
                JSONObject jsonObject = new JSONObject();
                Rescatalog rescatalog = resCatalogs.get(i);
                String has_child = rescatalog.getHasChild();
                if (("1").equals(has_child)) {
                    jsonObject.put("id", rescatalog.getId());
                    jsonObject.put("name", rescatalog.getAlias());
                    jsonObject.put("displayorder", rescatalog.getDisplayorder());
                    jsonObject.put("layers", getResListByPid(rescatalog.getId()));

                } else {
                    jsonObject.put("id", rescatalog.getId());
                    jsonObject.put("name", rescatalog.getName());
                    jsonObject.put("alias", rescatalog.getAlias());
                    jsonObject.put("url", rescatalog.getUrl());
                    jsonObject.put("sublayerid", rescatalog.getSublayerid());
                    jsonObject.put("dataport", rescatalog.getDataport());
                    jsonObject.put("relfield", rescatalog.getRelfield());
                    jsonObject.put("legend", rescatalog.getLegend());
                    jsonObject.put("visible", Boolean.parseBoolean(rescatalog.getVisible()));
                    jsonObject.put("labelsvisible", Boolean.parseBoolean(rescatalog.getLabelsvisible()));
                    jsonObject.put("analyzable", Boolean.parseBoolean(rescatalog.getAnalyzable()));
                    jsonObject.put("sourceName", rescatalog.getSourcename());
                    jsonObject.put("queryable", Boolean.parseBoolean(rescatalog.getQueryable()));
                    jsonObject.put("tablename", rescatalog.getTablename());
                    jsonObject.put("servicetype", rescatalog.getServicetype());
                    jsonObject.put("zIndex", rescatalog.getZindex());
                    jsonObject.put("displayorder", rescatalog.getDisplayorder());
                    //jsonObject.put("style", rescatalog.getStyle());

//					 //组装fields（改为从通用查询配置中获取）
//					 LambdaQueryWrapper<Layerfields> fieldsQueryWrapper = new LambdaQueryWrapper<>();
//					 fieldsQueryWrapper.eq(Layerfields::getLayername,rescatalog.getName());
//					 fieldsQueryWrapper.orderByAsc(Layerfields::getXh);
//					 List<Layerfields> layerFields = layerfieldsService.list(fieldsQueryWrapper);
//					 List<JSONObject> fieldList = new ArrayList<>();
//					 if(layerFields.size()>0){
//						 for(int j=0;j<layerFields.size();j++){
//							 JSONObject fieldJson = new JSONObject();
//							 Layerfields layerField = layerFields.get(j);
//							 fieldJson.put("name",layerField.getName());
//							 fieldJson.put("label",layerField.getAlias());
//							 fieldList.add(fieldJson);
//						 }
//					 }
//					 jsonObject.put("fields",fieldList);
//					 //组装categories
//					 LambdaQueryWrapper<Layercategories> categoriesQueryWrapper = new LambdaQueryWrapper<>();
//					 categoriesQueryWrapper.eq(Layercategories::getLayername,rescatalog.getName());
//					 List<Layercategories> layerCategories = layercategoriesService.list(categoriesQueryWrapper);
//					 List<JSONObject> categoryList = new ArrayList<>();
//					 if(layerCategories.size()>0){
//						 for(int j=0;j<layerCategories.size();j++){
//							 JSONObject categoryJson = new JSONObject();
//							 Layercategories layerCategory = layerCategories.get(j);
//							 categoryJson.put("name",layerCategory.getLegendname());
//							 categoryJson.put("img",layerCategory.getLegendimg());
//							 categoryList.add(categoryJson);
//						 }
//					 }
//					 jsonObject.put("categories",categoryList);
                }
                resList.add(jsonObject);
            }
        }
        return resList;
    }

    /**
     * 分页列表查询
     *
     * @param rescatalog
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AutoLog(value = "rescatalog-分页列表查询")
    @ApiOperation(value = "rescatalog-分页列表查询", notes = "rescatalog-分页列表查询")
    @GetMapping(value = "/rootList")
    public Result<IPage<Rescatalog>> queryPageList(Rescatalog rescatalog,
                                                   @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                   @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                   HttpServletRequest req) {
        String hasQuery = req.getParameter("hasQuery");
        if (hasQuery != null && "true".equals(hasQuery)) {
            QueryWrapper<Rescatalog> queryWrapper = QueryGenerator.initQueryWrapper(rescatalog, req.getParameterMap());
            List<Rescatalog> list = rescatalogService.queryTreeListNoPage(queryWrapper);
            IPage<Rescatalog> pageList = new Page<>(1, 10, list.size());
            pageList.setRecords(list);
            return Result.OK(pageList);
        } else {
            String parentId = rescatalog.getPid();
            if (oConvertUtils.isEmpty(parentId)) {
                parentId = "0";
            }
            rescatalog.setPid(null);
            QueryWrapper<Rescatalog> queryWrapper = QueryGenerator.initQueryWrapper(rescatalog, req.getParameterMap());
            // 使用 eq 防止模糊查询
            queryWrapper.eq("pid", parentId);
            Page<Rescatalog> page = new Page<Rescatalog>(pageNo, pageSize);
            IPage<Rescatalog> pageList = rescatalogService.page(page, queryWrapper);
            return Result.OK(pageList);
        }
    }

    /**
     * 【vue3专用】加载节点的子数据
     *
     * @param pid
     * @return
     */
    @RequestMapping(value = "/loadTreeChildren", method = RequestMethod.GET)
    public Result<List<SelectTreeModel>> loadTreeChildren(@RequestParam(name = "pid") String pid) {
        Result<List<SelectTreeModel>> result = new Result<>();
        try {
            List<SelectTreeModel> ls = rescatalogService.queryListByPid(pid);
            result.setResult(ls);
            result.setSuccess(true);
        } catch (Exception e) {
            e.printStackTrace();
            result.setMessage(e.getMessage());
            result.setSuccess(false);
        }
        return result;
    }

    /**
     * 【vue3专用】加载一级节点/如果是同步 则所有数据
     *
     * @param async
     * @param pcode
     * @return
     */
    @RequestMapping(value = "/loadTreeRoot", method = RequestMethod.GET)
    public Result<List<SelectTreeModel>> loadTreeRoot(@RequestParam(name = "async") Boolean async, @RequestParam(name = "pcode") String pcode) {
        Result<List<SelectTreeModel>> result = new Result<>();
        try {
            List<SelectTreeModel> ls = rescatalogService.queryListByCode(pcode);
            if (!async) {
                loadAllChildren(ls);
            }
            result.setResult(ls);
            result.setSuccess(true);
        } catch (Exception e) {
            e.printStackTrace();
            result.setMessage(e.getMessage());
            result.setSuccess(false);
        }
        return result;
    }

    /**
     * 【vue3专用】递归求子节点 同步加载用到
     *
     * @param ls
     */
    private void loadAllChildren(List<SelectTreeModel> ls) {
        for (SelectTreeModel tsm : ls) {
            List<SelectTreeModel> temp = rescatalogService.queryListByPid(tsm.getKey());
            if (temp != null && temp.size() > 0) {
                tsm.setChildren(temp);
                loadAllChildren(temp);
            }
        }
    }

    /**
     * 获取子数据
     *
     * @param rescatalog
     * @param req
     * @return
     */
    //@AutoLog(value = "rescatalog-获取子数据")
    @ApiOperation(value = "rescatalog-获取子数据", notes = "rescatalog-获取子数据")
    @GetMapping(value = "/childList")
    public Result<IPage<Rescatalog>> queryPageList(Rescatalog rescatalog, HttpServletRequest req) {
        QueryWrapper<Rescatalog> queryWrapper = QueryGenerator.initQueryWrapper(rescatalog, req.getParameterMap());
        List<Rescatalog> list = rescatalogService.list(queryWrapper);
        IPage<Rescatalog> pageList = new Page<>(1, 10, list.size());
        pageList.setRecords(list);
        return Result.OK(pageList);
    }

    /**
     * 批量查询子节点
     *
     * @param parentIds 父ID（多个采用半角逗号分割）
     * @param parentIds
     * @return 返回 IPage
     * @return
     */
    //@AutoLog(value = "rescatalog-批量获取子数据")
    @ApiOperation(value = "rescatalog-批量获取子数据", notes = "rescatalog-批量获取子数据")
    @GetMapping("/getChildListBatch")
    public Result getChildListBatch(@RequestParam("parentIds") String parentIds) {
        try {
            QueryWrapper<Rescatalog> queryWrapper = new QueryWrapper<>();
            List<String> parentIdList = Arrays.asList(parentIds.split(","));
            queryWrapper.in("pid", parentIdList);
            List<Rescatalog> list = rescatalogService.list(queryWrapper);
            IPage<Rescatalog> pageList = new Page<>(1, 10, list.size());
            pageList.setRecords(list);
            return Result.OK(pageList);
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            return Result.error("批量查询子节点失败：" + e.getMessage());
        }
    }

    /**
     * 添加
     *
     * @param rescatalog
     * @return
     */
    @AutoLog(value = "rescatalog-添加")
    @ApiOperation(value = "rescatalog-添加", notes = "rescatalog-添加")
    @PostMapping(value = "/add")
    public Result<String> add(@RequestBody Rescatalog rescatalog) {
        if(rescatalog.getSublayerid() != null){
            int subLayerId = rescatalog.getSublayerid();
            if (subLayerId > -1) {
                LambdaQueryWrapper<Rescatalog> queryWrapper = new LambdaQueryWrapper<>();
                queryWrapper.eq(Rescatalog::getSublayerid, subLayerId);
                List<Rescatalog> resCatalogList = rescatalogService.list(queryWrapper);
                if (resCatalogList.size() > 0) {
                    rescatalogService.subLayerIdAdditive(subLayerId);
                }
            }
        }
        rescatalogService.addRescatalog(rescatalog);
        return Result.OK("添加成功！");
    }

    /**
     * 编辑
     *
     * @param rescatalog
     * @return
     */
    @AutoLog(value = "rescatalog-编辑")
    @ApiOperation(value = "rescatalog-编辑", notes = "rescatalog-编辑")
    @RequestMapping(value = "/edit", method = {RequestMethod.PUT, RequestMethod.POST})
    public Result<String> edit(@RequestBody Rescatalog rescatalog) {
        rescatalogService.updateRescatalog(rescatalog);
        return Result.OK("编辑成功!");
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "rescatalog-通过id删除")
    @ApiOperation(value = "rescatalog-通过id删除", notes = "rescatalog-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<String> delete(@RequestParam(name = "id", required = true) String id) {
        Rescatalog rescatalog = rescatalogService.getById(id);
        if(rescatalog.getSublayerid() != null){
            int subLayerId = rescatalog.getSublayerid();
            if (subLayerId > -1) {
                rescatalogService.subLayerIdDecrement(subLayerId);
            }
        }
        rescatalogService.deleteRescatalog(id);
        return Result.OK("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "rescatalog-批量删除")
    @ApiOperation(value = "rescatalog-批量删除", notes = "rescatalog-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<String> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        this.rescatalogService.removeByIds(Arrays.asList(ids.split(",")));
        return Result.OK("批量删除成功！");
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    //@AutoLog(value = "rescatalog-通过id查询")
    @ApiOperation(value = "rescatalog-通过id查询", notes = "rescatalog-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<Rescatalog> queryById(@RequestParam(name = "id", required = true) String id) {
        Rescatalog rescatalog = rescatalogService.getById(id);
        if (rescatalog == null) {
            return Result.error("未找到对应数据");
        }
        return Result.OK(rescatalog);
    }

    /**
     * 导出excel
     *
     * @param request
     * @param rescatalog
     */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, Rescatalog rescatalog) {
        return super.exportXls(request, rescatalog, Rescatalog.class, "rescatalog");
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        return super.importExcel(request, response, Rescatalog.class);
    }

    /**
     * 用户角色授权功能，查询资源目录权限树
     *
     * @param request
     * @return
     */
    @RequestMapping(value = "/queryTreeList", method = RequestMethod.GET)
    public Result<Map<String, Object>> queryTreeList(HttpServletRequest request) {
        Result<Map<String, Object>> result = new Result<>();
        //全部资源目录ids
        List<String> ids = new ArrayList<>();
        try {
            LambdaQueryWrapper<Rescatalog> query = new LambdaQueryWrapper<>();
            query.orderByAsc(Rescatalog::getDisplayorder);
            List<Rescatalog> list = rescatalogService.list(query);
            for (Rescatalog rescatalog : list) {
                ids.add(rescatalog.getId());
            }
            List<TreeModel> treeList = new ArrayList<>();
            getTreeModelList(treeList, list, null);
            Map<String, Object> resMap = new HashMap<String, Object>();
            resMap.put("treeList", treeList); //全部树节点数据
            resMap.put("ids", ids);//全部树ids
            result.setResult(resMap);
            result.setSuccess(true);
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }
        return result;
    }

    private void getTreeModelList(List<TreeModel> treeList, List<Rescatalog> rescatalogList, TreeModel temp) {
        for (Rescatalog rescatalog : rescatalogList) {
            String tempPid = rescatalog.getPid();
            Integer ruleFlag = 0;
            Boolean isLeaf = rescatalog.getHasChild().equals("0") ? true : false;
            TreeModel tree = new TreeModel(rescatalog.getId(), tempPid, rescatalog.getAlias(), ruleFlag, isLeaf);
//			 if(temp==null && oConvertUtils.isEmpty(tempPid)) {
            if (temp == null && ("0".equals(tempPid))) {
                treeList.add(tree);
                if (!tree.getIsLeaf()) {
                    getTreeModelList(treeList, rescatalogList, tree);
                }
            } else if (temp != null && tempPid != null && tempPid.equals(temp.getKey())) {
                temp.getChildren().add(tree);
                if (!tree.getIsLeaf()) {
                    getTreeModelList(treeList, rescatalogList, tree);
                }
            }

        }
    }
}
