package org.jeecg.modules.resManager.categories.controller;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecg.modules.resManager.categories.entity.Layercategories;
import org.jeecg.modules.resManager.categories.service.ILayercategoriesService;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.extern.slf4j.Slf4j;

import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.jeecg.common.system.base.controller.JeecgController;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;
import com.alibaba.fastjson.JSON;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.jeecg.common.aspect.annotation.AutoLog;

 /**
 * @Description: 图层图例展示类型
 * @Author: jeecg-boot
 * @Date:   2022-01-17
 * @Version: V1.0
 */
@Api(tags="图层图例展示类型")
@RestController
@RequestMapping("/resManager.categories/layercategories")
@Slf4j
public class LayercategoriesController extends JeecgController<Layercategories, ILayercategoriesService> {
	@Autowired
	private ILayercategoriesService layercategoriesService;
	
	/**
	 * 分页列表查询
	 *
	 * @param layercategories
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	@AutoLog(value = "图层图例展示类型-分页列表查询")
	@ApiOperation(value="图层图例展示类型-分页列表查询", notes="图层图例展示类型-分页列表查询")
	@GetMapping(value = "/list")
	public Result<?> queryPageList(Layercategories layercategories,
								   @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
								   @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
								   HttpServletRequest req) {
		QueryWrapper<Layercategories> queryWrapper = QueryGenerator.initQueryWrapper(layercategories, req.getParameterMap());
		Page<Layercategories> page = new Page<Layercategories>(pageNo, pageSize);
		IPage<Layercategories> pageList = layercategoriesService.page(page, queryWrapper);
		return Result.OK(pageList);
	}
	
	/**
	 *   添加
	 *
	 * @param layercategories
	 * @return
	 */
	@AutoLog(value = "图层图例展示类型-添加")
	@ApiOperation(value="图层图例展示类型-添加", notes="图层图例展示类型-添加")
	@PostMapping(value = "/add")
	public Result<?> add(@RequestBody Layercategories layercategories) {
		layercategoriesService.save(layercategories);
		return Result.OK("添加成功！");
	}
	
	/**
	 *  编辑
	 *
	 * @param layercategories
	 * @return
	 */
	@AutoLog(value = "图层图例展示类型-编辑")
	@ApiOperation(value="图层图例展示类型-编辑", notes="图层图例展示类型-编辑")
	@PostMapping(value = "/edit")
	public Result<?> edit(@RequestBody Layercategories layercategories) {
		layercategoriesService.updateById(layercategories);
		return Result.OK("编辑成功!");
	}
	
	/**
	 *   通过id删除
	 *
	 * @param id
	 * @return
	 */
	@AutoLog(value = "图层图例展示类型-通过id删除")
	@ApiOperation(value="图层图例展示类型-通过id删除", notes="图层图例展示类型-通过id删除")
	@PostMapping(value = "/delete")
	public Result<?> delete(@RequestParam(name="id",required=true) String id) {
		layercategoriesService.removeById(id);
		return Result.OK("删除成功!");
	}
	
	/**
	 *  批量删除
	 *
	 * @param ids
	 * @return
	 */
	@AutoLog(value = "图层图例展示类型-批量删除")
	@ApiOperation(value="图层图例展示类型-批量删除", notes="图层图例展示类型-批量删除")
	@PostMapping(value = "/deleteBatch")
	public Result<?> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		this.layercategoriesService.removeByIds(Arrays.asList(ids.split(",")));
		return Result.OK("批量删除成功!");
	}
	
	/**
	 * 通过id查询
	 *
	 * @param id
	 * @return
	 */
	@AutoLog(value = "图层图例展示类型-通过id查询")
	@ApiOperation(value="图层图例展示类型-通过id查询", notes="图层图例展示类型-通过id查询")
	@GetMapping(value = "/queryById")
	public Result<?> queryById(@RequestParam(name="id",required=true) String id) {
		Layercategories layercategories = layercategoriesService.getById(id);
		if(layercategories==null) {
			return Result.error("未找到对应数据");
		}
		return Result.OK(layercategories);
	}

    /**
    * 导出excel
    *
    * @param request
    * @param layercategories
    */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, Layercategories layercategories) {
        return super.exportXls(request, layercategories, Layercategories.class, "图层图例展示类型");
    }

    /**
      * 通过excel导入数据
    *
    * @param request
    * @param response
    * @return
    */
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        return super.importExcel(request, response, Layercategories.class);
    }

}
