package org.jeecg.modules.toolManage.controller;

import java.util.*;
import java.util.stream.Collectors;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecg.common.system.vo.SelectTreeModel;
import org.jeecg.modules.base.model.TreeModel;
import org.jeecg.modules.resManager.catalog.entity.Rescatalog;
import org.jeecg.modules.toolManage.entity.Toolmanage;
import org.jeecg.modules.toolManage.service.IToolmanageService;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.extern.slf4j.Slf4j;

import org.jeecg.modules.topicManage.entity.TopicsConfig;
import org.jeecg.modules.topicManage.service.ITopicsConfigService;
import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.jeecg.common.system.base.controller.JeecgController;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;
import com.alibaba.fastjson.JSON;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.jeecg.common.aspect.annotation.AutoLog;

/**
 * @Description: 地图工具管理
 * @Author: jeecg-boot
 * @Date: 2022-07-30
 * @Version: V1.0
 */
@Api(tags = "地图工具管理")
@RestController
@RequestMapping("/toolManage/toolmanage")
@Slf4j
public class ToolmanageController extends JeecgController<Toolmanage, IToolmanageService> {
    @Autowired
    private IToolmanageService toolmanageService;
    @Autowired
    private ITopicsConfigService topicsConfigService;

    @AutoLog(value = "地图工具-地图工具查询")
    @ApiOperation(value = "地图工具-地图工具查询", notes = "地图工具-地图工具查询")
    @GetMapping(value = "/toolListByUser")
    public Result<?> getToolListByUserName(@RequestParam("username") String username, @RequestParam("topicId") String topicId) {
        //1.根据用户名获取当前用户权限下的所有工具
        List<Toolmanage> toolmanageList = toolmanageService.queryByUser(username);
        //2.根据传入的topicId，获取专题配置的所有工具
        String disptools = "";
        if (!"null".equals(topicId) && !topicId.isEmpty()) {
            LambdaQueryWrapper<TopicsConfig> topicsConfigLambdaQueryWrapper = new LambdaQueryWrapper<>();
            topicsConfigLambdaQueryWrapper.eq(TopicsConfig::getTopicid, topicId);
            TopicsConfig topicsConfig = topicsConfigService.getOne(topicsConfigLambdaQueryWrapper);
            disptools= topicsConfig.getDisptools();
        }
        //3.取两个集合的交集作为最终的工具
        List<String> toolList = new ArrayList<>();
        if (toolmanageList.size() > 0) {
            for (int i = 0; i < toolmanageList.size(); i++) {
                Toolmanage toolmanage = toolmanageList.get(i);
                String registcode = toolmanage.getRegistcode();
                if(disptools.isEmpty() || disptools.indexOf(registcode) !=-1){
                    toolList.add(registcode);
                }
            }
        }
        String[] tools = toolList.toArray(new String[toolList.size()]);
        return Result.OK(tools);
    }

    /**
     * 分页列表查询
     *
     * @param toolmanage
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    //@AutoLog(value = "地图工具管理-分页列表查询")
    @ApiOperation(value = "地图工具管理-分页列表查询", notes = "地图工具管理-分页列表查询")
    @GetMapping(value = "/rootList")
    public Result<IPage<Toolmanage>> queryPageList(Toolmanage toolmanage,
                                                   @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                   @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                   HttpServletRequest req) {
        String hasQuery = req.getParameter("hasQuery");
        if (hasQuery != null && "true".equals(hasQuery)) {
            QueryWrapper<Toolmanage> queryWrapper = QueryGenerator.initQueryWrapper(toolmanage, req.getParameterMap());
            List<Toolmanage> list = toolmanageService.queryTreeListNoPage(queryWrapper);
            IPage<Toolmanage> pageList = new Page<>(1, 10, list.size());
            pageList.setRecords(list);
            return Result.OK(pageList);
        } else {
            String parentId = toolmanage.getPid();
            if (oConvertUtils.isEmpty(parentId)) {
                parentId = "0";
            }
            toolmanage.setPid(null);
            QueryWrapper<Toolmanage> queryWrapper = QueryGenerator.initQueryWrapper(toolmanage, req.getParameterMap());
            // 使用 eq 防止模糊查询
            queryWrapper.eq("pid", parentId);
            Page<Toolmanage> page = new Page<Toolmanage>(pageNo, pageSize);
            IPage<Toolmanage> pageList = toolmanageService.page(page, queryWrapper);
            return Result.OK(pageList);
        }
    }

    /**
     * 【vue3专用】加载节点的子数据
     *
     * @param pid
     * @return
     */
    @RequestMapping(value = "/loadTreeChildren", method = RequestMethod.GET)
    public Result<List<SelectTreeModel>> loadTreeChildren(@RequestParam(name = "pid") String pid) {
        Result<List<SelectTreeModel>> result = new Result<>();
        try {
            List<SelectTreeModel> ls = toolmanageService.queryListByPid(pid);
            result.setResult(ls);
            result.setSuccess(true);
        } catch (Exception e) {
            e.printStackTrace();
            result.setMessage(e.getMessage());
            result.setSuccess(false);
        }
        return result;
    }

    /**
     * 【vue3专用】加载一级节点/如果是同步 则所有数据
     *
     * @param async
     * @param pcode
     * @return
     */
    @RequestMapping(value = "/loadTreeRoot", method = RequestMethod.GET)
    public Result<List<SelectTreeModel>> loadTreeRoot(@RequestParam(name = "async") Boolean async, @RequestParam(name = "pcode") String pcode) {
        Result<List<SelectTreeModel>> result = new Result<>();
        try {
            List<SelectTreeModel> ls = toolmanageService.queryListByCode(pcode);
            if (!async) {
                loadAllChildren(ls);
            }
            result.setResult(ls);
            result.setSuccess(true);
        } catch (Exception e) {
            e.printStackTrace();
            result.setMessage(e.getMessage());
            result.setSuccess(false);
        }
        return result;
    }

    /**
     * 【vue3专用】递归求子节点 同步加载用到
     *
     * @param ls
     */
    private void loadAllChildren(List<SelectTreeModel> ls) {
        for (SelectTreeModel tsm : ls) {
            List<SelectTreeModel> temp = toolmanageService.queryListByPid(tsm.getKey());
            if (temp != null && temp.size() > 0) {
                tsm.setChildren(temp);
                loadAllChildren(temp);
            }
        }
    }

    /**
     * 获取子数据
     *
     * @param toolmanage
     * @param req
     * @return
     */
    //@AutoLog(value = "地图工具管理-获取子数据")
    @ApiOperation(value = "地图工具管理-获取子数据", notes = "地图工具管理-获取子数据")
    @GetMapping(value = "/childList")
    public Result<IPage<Toolmanage>> queryPageList(Toolmanage toolmanage, HttpServletRequest req) {
        QueryWrapper<Toolmanage> queryWrapper = QueryGenerator.initQueryWrapper(toolmanage, req.getParameterMap());
        List<Toolmanage> list = toolmanageService.list(queryWrapper);
        IPage<Toolmanage> pageList = new Page<>(1, 10, list.size());
        pageList.setRecords(list);
        return Result.OK(pageList);
    }

    /**
     * 批量查询子节点
     *
     * @param parentIds 父ID（多个采用半角逗号分割）
     * @param parentIds
     * @return 返回 IPage
     * @return
     */
    //@AutoLog(value = "地图工具管理-批量获取子数据")
    @ApiOperation(value = "地图工具管理-批量获取子数据", notes = "地图工具管理-批量获取子数据")
    @GetMapping("/getChildListBatch")
    public Result getChildListBatch(@RequestParam("parentIds") String parentIds) {
        try {
            QueryWrapper<Toolmanage> queryWrapper = new QueryWrapper<>();
            List<String> parentIdList = Arrays.asList(parentIds.split(","));
            queryWrapper.in("pid", parentIdList);
            List<Toolmanage> list = toolmanageService.list(queryWrapper);
            IPage<Toolmanage> pageList = new Page<>(1, 10, list.size());
            pageList.setRecords(list);
            return Result.OK(pageList);
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            return Result.error("批量查询子节点失败：" + e.getMessage());
        }
    }

    /**
     * 添加
     *
     * @param toolmanage
     * @return
     */
    @AutoLog(value = "地图工具管理-添加")
    @ApiOperation(value = "地图工具管理-添加", notes = "地图工具管理-添加")
    @PostMapping(value = "/add")
    public Result<String> add(@RequestBody Toolmanage toolmanage) {
        toolmanageService.addToolmanage(toolmanage);
        return Result.OK("添加成功！");
    }

    /**
     * 编辑
     *
     * @param toolmanage
     * @return
     */
    @AutoLog(value = "地图工具管理-编辑")
    @ApiOperation(value = "地图工具管理-编辑", notes = "地图工具管理-编辑")
    @RequestMapping(value = "/edit", method = {RequestMethod.PUT, RequestMethod.POST})
    public Result<String> edit(@RequestBody Toolmanage toolmanage) {
        toolmanageService.updateToolmanage(toolmanage);
        return Result.OK("编辑成功!");
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "地图工具管理-通过id删除")
    @ApiOperation(value = "地图工具管理-通过id删除", notes = "地图工具管理-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<String> delete(@RequestParam(name = "id", required = true) String id) {
        toolmanageService.deleteToolmanage(id);
        return Result.OK("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "地图工具管理-批量删除")
    @ApiOperation(value = "地图工具管理-批量删除", notes = "地图工具管理-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<String> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        this.toolmanageService.removeByIds(Arrays.asList(ids.split(",")));
        return Result.OK("批量删除成功！");
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    //@AutoLog(value = "地图工具管理-通过id查询")
    @ApiOperation(value = "地图工具管理-通过id查询", notes = "地图工具管理-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<Toolmanage> queryById(@RequestParam(name = "id", required = true) String id) {
        Toolmanage toolmanage = toolmanageService.getById(id);
        if (toolmanage == null) {
            return Result.error("未找到对应数据");
        }
        return Result.OK(toolmanage);
    }

    /**
     * 导出excel
     *
     * @param request
     * @param toolmanage
     */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, Toolmanage toolmanage) {
        return super.exportXls(request, toolmanage, Toolmanage.class, "地图工具管理");
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        return super.importExcel(request, response, Toolmanage.class);
    }

    /**
     * 用户角色授权功能，查询地图工具权限树
     *
     * @param request
     * @return
     */
    @RequestMapping(value = "/queryTreeList", method = RequestMethod.GET)
    public Result<Map<String, Object>> queryTreeList(HttpServletRequest request) {
        Result<Map<String, Object>> result = new Result<>();
        //全部资源目录ids
        List<String> ids = new ArrayList<>();
        try {
            LambdaQueryWrapper<Toolmanage> query = new LambdaQueryWrapper<>();
            List<Toolmanage> list = toolmanageService.list(query);
            for (Toolmanage toolmanage : list) {
                ids.add(toolmanage.getId());
            }
            List<TreeModel> treeList = new ArrayList<>();
            getTreeModelList(treeList, list, null);
            Map<String, Object> toolMap = new HashMap<String, Object>();
            toolMap.put("treeList", treeList); //全部树节点数据
            toolMap.put("ids", ids);//全部树ids
            result.setResult(toolMap);
            result.setSuccess(true);
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }
        return result;
    }

    private void getTreeModelList(List<TreeModel> treeList, List<Toolmanage> toolmanageList, TreeModel temp) {
        for (Toolmanage toolmanage : toolmanageList) {
            String tempPid = toolmanage.getPid();
            Integer ruleFlag = 0;
            Boolean isLeaf = toolmanage.getHasChild().equals("0") ? true : false;
            TreeModel tree = new TreeModel(toolmanage.getId(), tempPid, toolmanage.getName(), ruleFlag, isLeaf);
//			 if(temp==null && oConvertUtils.isEmpty(tempPid)) {
            if (temp == null && ("0".equals(tempPid))) {
                treeList.add(tree);
                if (!tree.getIsLeaf()) {
                    getTreeModelList(treeList, toolmanageList, tree);
                }
            } else if (temp != null && tempPid != null && tempPid.equals(temp.getKey())) {
                temp.getChildren().add(tree);
                if (!tree.getIsLeaf()) {
                    getTreeModelList(treeList, toolmanageList, tree);
                }
            }

        }
    }
}
